from dataclasses import dataclass
from moodle import BaseNameValue


@dataclass
class ContentOption(BaseNameValue):
    """Options, used since Moodle 2.9

    Args:
        name (str): The expected keys (value format) are:
                    excludemodules (bool) Do not return modules, return only the sections structure
                    excludecontents (bool) Do not return module contents (i.e: files inside a resource)
                    includestealthmodules (bool) Return stealth modules for students in a special
                        section (with id -1)
                    sectionid (int) Return only this section
                    sectionnumber (int) Return only this section with number (order)
                    cmid (int) Return only this module information (among the whole sections structure)
                    modname (string) Return only modules with this name "label, forum, etc..."
                    modid (int) Return only the module with this id (to be used with modname
        value (str): the value of the option, this param is personaly validated in the external function.
    """
    pass



from .note import Note
from .site_note import SiteNote
from .course_note import CourseNote
from .personal_note import PersonalNote
from .course_notes import CourseNotes
from .view_notes import ViewNotes

from .base import BaseNotes

__all__ = [
    'Note',
    'SiteNote',
    'CourseNote',
    'PersonalNote',
    'CourseNotes',
    'ViewNotes',
    'BaseNotes',
]



from dataclasses import dataclass


@dataclass
class Period:
    """Period
    Constructor arguments:
    params: seconds (int): seconds
    params: minutes (int): minutes
    params: hours (int): hours
    params: mday (int): mday
    params: wday (int): wday
    params: mon (int): mon
    params: year (int): year
    params: yday (int): yday
    params: weekday (str): weekday
    params: month (str): month
    params: timestamp (int): timestamp
    """
    seconds: int
    minutes: int
    hours: int
    mday: int
    wday: int
    mon: int
    year: int
    yday: int
    weekday: str
    month: str
    timestamp: int



from dataclasses import dataclass
from typing import List
from .notification_processor import NotificationProcessor


@dataclass
class NotificationComponent:
    displayname: str
    preferencekey: str
    processors: List[NotificationProcessor]



from dataclasses import dataclass
from typing import List, Optional


@dataclass
class Filling:
    """Filling
    Args:
        criterionid (int): criterion id
        levelid (Optional[int]): level id
        remark (Optional[str]): remark
        remarkformat (Optional[int]): remark format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        enrichedbenchmark (Optional[float]): enrichedbenchmark
        enrichedbenchmarkstudent (Optional[float]): enrichedbenchmarkstudent
        enrichedbenchmarkstudents (Optional[float]): enrichedbenchmarkstudents
    """
    criterionid: int
    levelid: Optional[int]
    remark: Optional[str]
    remarkformat: Optional[int]
    enrichedbenchmark: Optional[float]
    enrichedbenchmarkstudent: Optional[float]
    enrichedbenchmarkstudents: Optional[float]


@dataclass
class Criterion:
    """Criterion
    Args:
        criterionid (int): criterion id
        fillings (List[Filling]): filling
    """
    criterionid: int
    fillings: List[Filling]


@dataclass
class Rubric:
    """Rubric
    Args:
        criteria (List[Criterion]): list of criterion
    """
    criteria: List[Criterion]


@dataclass
class AdvanceGradingData:
    """Advance Grading Data
    Args:
        erubric (Optional[Rubric]): list of Rubric
        guide (Optional[Rubric]): list of Rubric
        rubric (Optional[Rubric]): list of Rubric
    """
    erubric: Optional[Rubric]
    guide: Optional[Rubric]
    rubric: Optional[Rubric]



from dataclasses import dataclass
from typing import Iterator, List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class CourseUpdate:
    name: str  # Name of the area updated.
    timeupdated: Optional[int]  # Last time was updated
    itemids: List[int]  # The ids of the items updated

    def __len__(self) -> int:
        return len(self.itemids)

    def __iter__(self) -> Iterator[int]:
        return iter(self.itemids)


@dataclass
class UpdateInstance:
    contextlevel: str  # The context level
    id: int  # Instance id
    updates: List[CourseUpdate]

    def __len__(self) -> int:
        return len(self.updates)

    def __iter__(self) -> Iterator[CourseUpdate]:
        return iter(self.updates)

    def __getitem__(self, slice: int) -> CourseUpdate:
        return self.updates[slice]


@dataclass
class CheckUpdate(ResponsesFactory[UpdateInstance]):
    instances: List[UpdateInstance]
    warnings: List[Warning]

    @property
    def items(self) -> List[UpdateInstance]:
        return self.instances

    @dataclass
    class ToCheck:
        """Instances to check
        Constructor arguments:
        params: contextlevel (str): The context level for the file location. Only module supported right now.
        params: id (int): Context instance id
        params: since (int): Check updates since this time stamp
        """
        contextlevel: str
        id: int
        since: int



from moodle import BaseMoodle
from . import BaseEmail


class Auth(BaseMoodle):
    def __post_init__(self, moodle) -> None:
        self._email = BaseEmail(moodle)

    @property
    def email(self) -> BaseEmail:
        return self._email

    @property
    def confirm_user(self):
        data = self.moodle.get('core_auth_confirm_user')
        return data

    @property
    def is_age_digital_consent_verification_enabled(self):
        data = self.moodle.get(
            'core_auth_is_age_digital_consent_verification_enabled')
        return data

    @property
    def is_minor(self):
        data = self.moodle.get('core_auth_is_minor')
        return data

    @property
    def request_password_reset(self):
        data = self.moodle.get('core_auth_request_password_reset')
        return data

    @property
    def resend_confirmation_email(self):
        data = self.moodle.get('core_auth_resend_confirmation_email')
        return data



from moodle import BaseMoodle


class BaseEmail(BaseMoodle):
    def get_signup_setting(self):
        # TODO auth_email_get_signup_setting
        raise NotImplementedError(
            'auth_email_get_signup_setting not implemented yet')
        data = self.moodle.get('auth_email_get_signup_setting')
        return data

    def signup_user(self):
        # TODO auth_email_get_signup_setting
        raise NotImplementedError(
            'auth_email_get_signup_setting not implemented yet')
        data = self.moodle.get('auth_email_signup_user')
        return data



from dataclasses import dataclass
from typing import List
from .advanced_feature import AdvancedFeatures
from .function import Function


@dataclass
class SiteInfo:
    sitename: str
    username: str
    firstname: str
    lastname: str
    fullname: str
    lang: str
    userid: int
    siteurl: str
    userpictureurl: str
    functions: List[Function]
    downloadfiles: int
    uploadfiles: int
    release: str
    version: str
    mobilecssurl: str
    advancedfeatures: List[AdvancedFeatures]
    usercanmanageownfiles: bool
    userquota: int
    usermaxuploadfilesize: int
    userhomepage: int
    siteid: int
    sitecalendartype: str
    usercalendartype: str
    theme: str



from moodle import BaseMoodle, UserPreference
from moodle.utils.helper import from_dict


class BaseUser(BaseMoodle):
    def agree_site_policy(self) -> bool:
        data = self.moodle.get('core_user_agree_site_policy')
        return data['status']

    def get_user_preferences(self) -> UserPreference:
        data = self.moodle.get('core_user_get_user_preferences')
        return from_dict(UserPreference, data)



from dataclasses import dataclass
from typing import Optional


@dataclass
class MoodleException(Exception):
    errorcode: Optional[str] = ''
    exception: Optional[str] = ''
    message: Optional[str] = ''

    def __str__(self):
        return self.message or self.exception or self.errorcode


@dataclass
class BaseException(Exception):
    msg: str = ''

    def __post_init__(self) -> None:
        self.msg = self.msg.capitalize()

    def __str__(self) -> str:
        return self.msg


@dataclass
class InvalidCredentialException(Exception):
    msg: str = 'Wrong username or password!'



from dataclasses import dataclass, field
from typing import List
from .notification_component import NotificationComponent


@dataclass
class PreferenceComponent:
    displayname: str
    notification: List[NotificationComponent] = field(default_factory=list)



from dataclasses import dataclass, field
from typing import List
from moodle import Warning
from .preference_component import PreferenceComponent
from .preference_processor import PreferenceProcessor
from .preference_user import PreferenceUser


@dataclass
class Preference:
    userid: int
    disableall: int
    processors: List[PreferenceProcessor] = field(default_factory=list)
    components: List[PreferenceComponent] = field(default_factory=list)


@dataclass
class NotificationPreference:
    preferences: Preference
    warnings: List[Warning] = field(default_factory=list)


@dataclass
class MessagePreference:
    preferences: Preference
    blocknoncontacts: int
    entertosend: bool
    warnings: List[Warning] = field(default_factory=list)


@dataclass
class UserPreference:
    prefereces: List[PreferenceUser] = field(default_factory=list)
    warnings: List[Warning] = field(default_factory=list)



from datetime import datetime
from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning


@dataclass
class Outcome:
    """Outcome information
    Args:
        id (str): Outcome id
        name (str): Outcome full name
        scale (str): Scale items
    """
    id: str
    name: str
    scale: str


@dataclass
class AdvancedGrading:
    """Advanced Grading
    Args:
        area (str):Gradable area name
        method (str):Grading method
    """
    area: str
    method: str


@dataclass
class Cm:
    """Course Module
    Args:
        id (int): The course module id
        course (int): The course id
        module (int): The module type id
        name (str): The activity name
        modname (str): The module component name (forum, assign, etc..)
        instance (int): The activity instance id
        section (int): The module section id
        sectionnum (int): The module section number
        groupmode (int): Group mode
        groupingid (int): Grouping id
        completion (int): If completion is enabled
        idnumber (Optional[str]): Module id number
        added (Optional[datetime]): Time added
        score (Optional[int]): Score
        indent (Optional[int]): Indentation
        visible (Optional[int]): If visible
        visibleoncoursepage (Optional[int]): If visible on course page
        visibleold (Optional[int]): Visible old
        completiongradeitemnumber (Optional[int]): Completion grade item
        completionview (Optional[int]): Completion view setting
        completionexpected (Optional[int]): Completion time expected
        showdescription (Optional[int]): If the description is showed
        availability (Optional[str]): Availability settings
        grade (Optional[float]): Grade (max value or scale id)
        scale (Optional[str]): Scale items (if used)
        gradepass (Optional[str]): Grade to pass (float)
        gradecat (Optional[int]): Grade category
        advancedgrading (List[AdvancedGrading]): Advanced grading settings
        outcomes (List[Outcome]): Outcomes information
    """
    id: int
    course: int
    module: int
    name: str
    modname: str
    instance: int
    section: int
    sectionnum: int
    groupmode: int
    groupingid: int
    completion: int
    idnumber: Optional[str]
    added: Optional[datetime]
    score: Optional[int]
    indent: Optional[int]
    visible: Optional[int]
    visibleoncoursepage: Optional[int]
    visibleold: Optional[int]
    completiongradeitemnumber: Optional[int]
    completionview: Optional[int]
    completionexpected: Optional[int]
    showdescription: Optional[int]
    availability: Optional[str]
    grade: Optional[float]
    scale: Optional[str]
    gradepass: Optional[str]
    gradecat: Optional[int]
    advancedgrading: List[AdvancedGrading]
    outcomes: List[Outcome]

    def __str__(self) -> str:
        return self.name


@dataclass
class CourseModule:
    cm: Cm
    warnings: List[Warning]



from dataclasses import dataclass
from typing import List
from moodle import Warning


@dataclass
class MobilePublicConfig:
    wwwroot: str
    httpswwwroot: str
    sitename: str
    guestlogin: int
    rememberusername: int
    authloginviaemail: int
    registerauth: str
    forgottenpasswordurl: str
    authinstructions: str
    authnoneenabled: int
    enablewebservices: int
    enablemobilewebservice: int
    maintenanceenabled: int
    maintenancemessage: str
    logourl: str
    compactlogourl: str
    typeoflogin: int
    launchurl: str
    mobilecssurl: str
    tool_mobile_disabledfeatures: str
    country: str
    agedigitalconsentverification: bool
    autolang: int
    lang: str
    langmenu: int
    langlist: str
    locale: str
    warnings: List[Warning]



from moodle import BaseMoodle
from . import BaseAssign, BaseForum


class Mod(BaseMoodle):
    def __post_init__(self, moodle) -> None:
        self._assign = BaseAssign(moodle)
        self._forum = BaseForum(moodle)

    @property
    def assign(self) -> BaseAssign:
        return self._assign

    @property
    def forum(self) -> BaseForum:
        return self._forum



from dataclasses import dataclass
from typing import List, Optional
from . import CourseEvent, DateView, Period


@dataclass
class DayView:
    """Day View
    Constructor arguments:
    params: events (List[CourseEvent]): events
    params: defaulteventcontext (int): defaulteventcontext
    params: filter_selector (str): filter_selector
    params: courseid (int): courseid
    params: categoryid (Optional[int]): categoryid
    params: neweventtimestamp (int): neweventtimestamp
    params: date (DateView): DateView
    params: periodname (str): periodname
    params: previousperiod (Period): Period
    params: nextperiodname (str): nextperiodname
    params: nextperiodlink (str): nextperiodlink
    params: nextperiod (Period): Period
    params: larrow (str): larrow
    params: rarrow (str): rarrow
    """
    events: List[CourseEvent]  # events
    defaulteventcontext: int  # defaulteventcontext
    filter_selector: str  # filter_selector
    courseid: int  # courseid
    categoryid: Optional[int]  # categoryid
    neweventtimestamp: int  # neweventtimestamp
    date: DateView
    periodname: str  # periodname
    previousperiod: Period
    nextperiodname: str  # nextperiodname
    nextperiodlink: str  # nextperiodlink
    nextperiod: Period
    larrow: str  # larrow
    rarrow: str  # rarrow



from datetime import datetime
from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class Update:
    name: str  # Name of the area updated.
    timeupdated: Optional[datetime]  # Last time was updated
    itemids: List[int]  # The ids of the items updated


@dataclass
class Instance:
    contextlevel: str  # The context level
    id: int  # Instance id
    updates: List[Update]  # updated area


@dataclass
class ActivityOverview(ResponsesFactory[Instance]):
    instances: List[Instance]  # list of instance
    warnings: List[Warning]  # list warning

    @property
    def items(self) -> List[Instance]:
        return self.instances



from dataclasses import dataclass
from typing import Optional


@dataclass
class Note:
    """Note to create
    Constructor arguments:
    params: userid (int): id of the user the note is about
    params: publishstate (str): 'personal', 'course' or 'site'
    params: courseid (int): course id of the note (in Moodle a note can only be created into a course, even for site and personal notes)
    params: text (str): the text of the message - text or HTML
    params: format (int): Default untuk "1" text format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: clientnoteid (Optional[str]): your own client id for the note. If this id is provided, the fail message id will be returned to you
    """
    userid: int
    publishstate: str
    courseid: int
    text: str
    format: int = 1
    clientnoteid: Optional[str] = None

    @dataclass
    class Result:
        """Result when creating notes
        Constructor arguments:
        params: clientnoteid (Optional[str]): your own id for the note
        params: noteid (int): ID of the created note when successful, -1 when failed
        params: errormessage (Optional[str]): error message - if failed
        """
        clientnoteid: Optional[str]
        noteid: int
        errormessage: Optional[str]



from dataclasses import dataclass
from datetime import datetime


@dataclass
class CourseNote:
    """Course Note
    Constructor arguments:
    params: id (int): id of this note
    params: courseid (int): id of the course
    params: userid (int): user id
    params: content (str): the content text formated
    params: format (int): content format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: created (int): time created (timestamp)
    params: lastmodified (int): time of last modification (timestamp)
    params: usermodified (int): user id of the creator of this note
    params: publishstate (str): state of the note (i.e. draft, public, site)
    """
    id: int
    courseid: int
    userid: int
    content: str
    format: int
    created: int
    lastmodified: datetime
    usermodified: int
    publishstate: str



from typing import List, Optional
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import BlogEntries, ViewEntry

Filter = BlogEntries.Filter


class BaseBlog(BaseMoodle):
    def get_entries(self,
                    filters: Optional[List[Filter]] = None,
                    page: int = 0,
                    perpage: int = 10) -> BlogEntries:
        """Get blog entries.

        Args:
            filters (Optional[List[Filter]], optional): Parameters to filter blog listings. Defaults to None.
            page (int, optional): The blog page to return. Defaults to 0.
            perpage (int, optional): The number of posts to return per page. Defaults to 10.

        Returns:
            BlogEntries: Returns blog entries.
        """
        res = self.moodle.post('core_blog_get_entries',
                               filters=filters or [],
                               page=page,
                               perpage=perpage)
        return from_dict(BlogEntries, res)

    def view_entries(self,
                     filters: Optional[List[Filter]] = None) -> ViewEntry:
        """Trigger the blog_entries_viewed event.

        Args:
            filters (Optional[List[Filter]], optional): Parameters used in the filter of view_entries.. Defaults to None.

        Returns:
            ViewEntry: the blog_entries_viewed response.
        """
        res = self.moodle.post('core_blog_view_entries', filters=filters)
        return from_dict(ViewEntry, res)



from __future__ import annotations
from dataclasses import dataclass
from typing import List, Optional
from moodle import MoodleObject


@dataclass
class ForumFile:
    """File in Forum
    Constructor arguments:
    params: filename (Optional[str]): File name.
    params: filepath (Optional[str]): File path.
    params: filesize (Optional[int]): File size.
    params: fileurl (Optional[str]): Downloadable file url.
    params: timemodified (Optional[int]): Time modified.
    params: mimetype (Optional[str]): File mime type.
    params: isexternalfile (Optional[int]): Whether is an external file.
    params: repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Forum(MoodleObject):
    """Forum
    Constructor arguments:
    params: id (int): Forum id
    params: course (int): Course id
    params: type (str): The forum type
    params: name (str): Forum name
    params: intro (str): The forum intro
    params: introformat (int): intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: duedate (Optional[int]): duedate for the user
    params: cutoffdate (Optional[int]): cutoffdate for the user
    params: assessed (int): Aggregate type
    params: assesstimestart (int): Assess start time
    params: assesstimefinish (int): Assess finish time
    params: scale (int): Scale
    params: maxbytes (int): Maximum attachment size
    params: maxattachments (int): Maximum number of attachments
    params: forcesubscribe (int): Force users to subscribe
    params: trackingtype (int): Subscription mode
    params: rsstype (int): RSS feed for this activity
    params: rssarticles (int): Number of RSS recent articles
    params: timemodified (int): Time modified
    params: warnafter (int): Post threshold for warning
    params: blockafter (int): Post threshold for blocking
    params: blockperiod (int): Time period for blocking
    params: completiondiscussions (int): Student must create discussions
    params: completionreplies (int): Student must post replies
    params: completionposts (int): Student must post discussions or replies
    params: cmid (int): Course module id
    params: numdiscussions (Optional[int]): Number of discussions in the forum
    params: cancreatediscussions (Optional[int]): If the user can create discussions
    params: lockdiscussionafter (Optional[int]): After what period a discussion is locked
    params: istracked (Optional[int]): If the user is tracking the forum
    params: unreadpostscount (Optional[int]): The number of unread posts for tracked forums
    """
    id: int
    course: int
    type: str
    name: str
    intro: str
    introformat: int
    introfiles: List[ForumFile]
    duedate: Optional[int]
    cutoffdate: Optional[int]
    assessed: int
    assesstimestart: int
    assesstimefinish: int
    scale: int
    maxbytes: int
    maxattachments: int
    forcesubscribe: int
    trackingtype: int
    rsstype: int
    rssarticles: int
    timemodified: int
    warnafter: int
    blockafter: int
    blockperiod: int
    completiondiscussions: int
    completionreplies: int
    completionposts: int
    cmid: int
    numdiscussions: Optional[int]
    cancreatediscussions: Optional[int]
    lockdiscussionafter: Optional[int]
    istracked: Optional[int]
    unreadpostscount: Optional[int]

    def __str__(self) -> str:
        return self.name



from dataclasses import dataclass
from typing import List
from moodle import Warning


@dataclass
class AccessInformation:
    """Permissions/access information
    Constructor arguments:
    params: canmanageentries (int): Whether the user can manage entries.
    params: canmanageownentries (int): Whether the user can manage its own entries.
    params: canmanagegroupentries (int): Whether the user can manage group entries.
    params: warnings (List[Warning]): list of warnings
    """
    canmanageentries: int
    canmanageownentries: int
    canmanagegroupentries: int
    warnings: List[Warning]



from dataclasses import dataclass, field
from typing import List, Optional
from moodle import BaseNameValue


@dataclass
class Restrict:
    """Restrict this content to certain users or courses.
    Constructor arguments:
    params: users (List[int]): List of allowed users.
    params: courses (List[int]): List of allowed courses.
    """
    users: List[int] = field(default_factory=list)
    courses: List[int] = field(default_factory=list)


@dataclass
class File:
    """File in the content.
    Constructor arguments:
    params: filename (Optional[str]): File name.
    params: filepath (Optional[str]): File path.
    params: filesize (Optional[int]): File size.
    params: fileurl (Optional[str]): Downloadable file url.
    params: timemodified (Optional[int]): Time modified.
    params: mimetype (Optional[str]): File mime type.
    params: isexternalfile (Optional[int]): Whether is an external file.
    params: repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class OtherData(BaseNameValue):
    """Other data that can be used or manipulated by the template via 2-way data-binding.
    Constructor arguments:
    params: name (str): Field name.
    params: value (str): Field value.
    """
    pass


@dataclass
class Template:
    """Template required by the generated content.
    Constructor arguments:
    params: id (str): ID of the template.
    params: html (str): HTML code.
    """
    id: str
    html: str


@dataclass
class Content:
    """A piece of content to be displayed in the Mobile app.
    Constructor arguments:
    params: templates (List[Template]): Templates required by the generated content.
    params: javascript (str): JavaScript code.
    params: otherdata (List[OtherData]): Other data that can be used or manipulated by the template via 2-way data-binding.
    params: files (List[File]): Files in the content.
    params: restrict (List[Restrict]): Restrict this content to certain users or courses.
    """
    templates: List[Template]
    javascript: str
    otherdata: List[OtherData]
    files: List[File]
    restrict: List[Restrict]

    @dataclass
    class Args(BaseNameValue):
        """Args for the method are optional.
        Constructor arguments:
        params: name (str): Param name.
        params: value (str): Param value.
        """
        pass



from typing import Optional
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import Blocks


class BaseBlock(BaseMoodle):
    def get_course_blocks(self,
                          courseid: int,
                          returncontents: Optional[int] = None) -> Blocks:
        """Blocks information for a course.

        Args:
            courseid (int): course id
            returncontents (Optional[int], optional): Whether to return the block contents.. Defaults to None.

        Returns:
            Blocks: Returns blocks information for a course.
        """
        res = self.moodle.post('core_block_get_course_blocks',
                               courseid=courseid,
                               returncontents=returncontents)
        return from_dict(Blocks, res)

    def get_dashboard_blocks(self,
                             userid: int = 0,
                             returncontents: bool = True) -> Blocks:
        """blocks information for the given user dashboard.

        Args:
            userid (int, optional): User id, default is current user. Defaults to 0.
            returncontents (bool, optional): Whether to return the block contents. Defaults to True.

        Returns:
            Blocks: Returns blocks information for the given user dashboard.
        """
        res = self.moodle.post('core_block_get_dashboard_blocks',
                               userid=userid,
                               returncontents=1 if returncontents else 0)
        return from_dict(Blocks, res)



from __future__ import annotations
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from moodle import Moodle


class BaseMoodle(object):
    def __init__(self, moodle):
        self._moodle = moodle
        self.__post_init__(moodle)

    def __post_init__(self, moodle) -> None:
        pass

    @property
    def moodle(self) -> Moodle:
        return self._moodle



from __future__ import annotations
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import SiteInfo


class BaseWebservice(BaseMoodle):
    def get_site_info(self) -> SiteInfo:
        data = self.moodle.get('core_webservice_get_site_info')
        if data:
            return from_dict(SiteInfo, data)
        raise Exception('Info not found')



from moodle import BaseMoodle


class BaseComment(BaseMoodle):
    def add_comments(self):
        data = self.moodle.get('core_comment_add_comments')
        return data

    def delete_comments(self):
        data = self.moodle.get('core_comment_delete_comments')
        return data

    def get_comments(self):
        data = self.moodle.get('core_comment_get_comments')
        return data



from .advance_grading_data import (
    Filling,
    Criterion,
    Rubric,
    AdvanceGradingData,
)
from .assignment import (
    Assignment,
    AssignmentCourse,
    Assignments,
)
from .plugin_data import PluginData
from .grade import (
    Grade,
    GradeAssignment,
    Grades,
)
from .participant import Participant
from .user_flag import UserFlag
from .view import View

from .base import BaseAssign

__all__ = [
    'Filling', 'Criterion', 'Rubric', 'AdvanceGradingData', 'Assignment',
    'AssignmentCourse', 'Assignments', 'Grade', 'GradeAssignment', 'Grades',
    'Participant', 'PluginData', 'UserFlag', 'View', 'BaseAssign'
]



from .badge import BagdeEndorsement, BagdeAlignment, Badge, BadgeResponse
from .base import BaseBadges

__all__ = [
    'BagdeEndorsement', 'BagdeAlignment', 'Badge', 'BadgeResponse',
    'BaseBadges'
]



from typing import List
from moodle import BaseMoodle, Warning
from moodle.utils.helper import from_dict
from . import Note, CourseNotes, ViewNotes


class BaseNotes(BaseMoodle):
    def create_notes(self, notes: List[Note]) -> List[Note.Result]:
        """Create notes

        Args:
            notes (List[Note]): Notes to create

        Returns:
            List[Note.Result]: Create note result
        """
        res = self.moodle.post('core_notes_create_notes', notes=notes)
        return [from_dict(Note.Result, data) for data in res] if res else []

    def delete_notes(self, notes: List[int]) -> List[Warning]:
        """Delete notes

        Args:
            notes (List[int]): Array of Note Ids to be deleted.

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post('core_notes_delete_notes', notes=notes)
        return [from_dict(Warning, data) for data in res] if res else []

    def get_course_notes(self, courseid: int, userid: int = 0) -> CourseNotes:
        """Returns all notes in specified course (or site), for the specified user.

        Args:
            courseid (int): course id, 0 for SITE
            userid (int, optional): user id. Defaults to 0.

        Returns:
            CourseNotes: Course note object
        """
        res = self.moodle.post('core_notes_get_course_notes',
                               courseid=courseid,
                               userid=userid)
        return from_dict(CourseNotes, res)

    def view_notes(self, courseid: int, userid: int = 0) -> ViewNotes:
        """Simulates the web interface view of notes/index.php: trigger events.

        Args:
            courseid (int): course id, 0 for notes at system level
            userid (int, optional): user id, 0 means view all the user notes. Defaults to 0.

        Returns:
            ViewNotes: view notes response
        """
        res = self.moodle.post('core_notes_view_notes',
                               courseid=courseid,
                               userid=userid)
        return from_dict(ViewNotes, res)



from typing import List, Optional
from moodle import BaseMoodle
from . import Forum


class BaseForum(BaseMoodle):
    def get_forums_by_courses(self,
                              courseids: Optional[List[int]] = None
                              ) -> List[Forum]:
        datas = self.moodle.get('mod_forum_get_forums_by_courses',
                                courseids=courseids or [])
        return [Forum.from_data(data, self.moodle)
                for data in datas] if datas else []



from dacite import from_dict as dacite_from_dict
from dacite.data import Data
from dataclasses import is_dataclass, asdict
from datetime import datetime
from typing import Type, TypeVar, Any
from moodle.config import DACITE_CONFIG

T = TypeVar("T")


def make_params(wstoken: str,
                wsfunction: str,
                moodlewsrestformat: str = 'json') -> dict:
    return {
        'wstoken': wstoken,
        'wsfunction': wsfunction,
        'moodlewsrestformat': moodlewsrestformat
    }


def from_dict(data_class: Type[T], data: Data) -> T:
    return dacite_from_dict(data_class, data, config=DACITE_CONFIG)


def to_dict(data: Any) -> Any:
    if not data:
        return data
    if isinstance(data, list):
        return [to_dict(d) for d in data]
    if isinstance(data, dict):
        out = {}
        for key, value in data.items():
            out[key] = to_dict(value)
        return out
    if is_dataclass(data):
        return asdict(data)
    if isinstance(data, datetime):
        return datetime.timestamp(data)
    return data



from dataclasses import dataclass, field
from typing import List
from moodle import Warning


@dataclass
class Key:
    """Autologin key
    params: key (str): Auto-login key for a single usage with time expiration.
    params: autologinurl (str): Auto-login URL.
    params: warnings (List[Warning]): list of warnings
    """
    key: str
    autologinurl: str
    warnings: List[Warning] = field(default_factory=list)

    def __str__(self) -> str:
        return self.key



from dataclasses import dataclass
from typing import Optional, List
from moodle import Warning, ResponsesFactory


@dataclass
class Event:
    """Event
    Constructor arguments:
    params: id (int): event id
    params: name (str): event name
    params: description (Optional[str]): Description
    params: format (int): description format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: courseid (int): course id
    params: categoryid (Optional[int]): Category id (only for category events).
    params: groupid (int): group id
    params: userid (int): user id
    params: repeatid (int): repeat id
    params: modulename (Optional[str]): module name
    params: instance (int): instance id
    params: eventtype (str): Event type
    params: timestart (int): timestart
    params: timeduration (int): time duration
    params: visible (int): visible
    params: uuid (Optional[str]): unique id of ical events
    params: sequence (int): sequence
    params: timemodified (int): time modified
    params: subscriptionid (Optional[int]): Subscription id
    """
    id: int
    name: str
    description: Optional[str]
    format: int
    courseid: int
    categoryid: Optional[int]
    groupid: Optional[int]
    userid: int
    repeatid: int
    modulename: Optional[str]
    instance: Optional[int]
    eventtype: str
    timestart: int
    timeduration: int
    visible: int
    uuid: Optional[str]
    sequence: int
    timemodified: int
    subscriptionid: Optional[int]

    def __str__(self) -> str:
        return self.name


@dataclass
class Events(ResponsesFactory[Event]):
    events: List[Event]
    warnings: List[Warning]

    @property
    def items(self) -> List[Event]:
        return self.events

    @dataclass
    class Create:
        """Event to create
        Constructor arguments:
        params: name (str): event name
        params: description (Optional[str]): Default for "null" Description
        params: format (Optional[int]): Default for "1" description format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        params: courseid (Optional[int]): Default for "0" course id
        params: groupid (Optional[int]): Default for "0" group id
        params: repeats (Optional[int]): Default for "0" number of repeats
        params: eventtype (Optional[str]): Default for "user" Event type
        params: timestart (Optional[int]): Default for "1599354090" timestart
        params: timeduration (Optional[int]): Default for "0" time duration
        params: visible (Optional[int]): Default for "1" visible
        params: sequence (Optional[int]): Default for "1" sequence
        """
        name: str
        description: Optional[str] = None
        format: Optional[int] = 1
        courseid: Optional[int] = 0
        groupid: Optional[int] = 0
        repeats: Optional[int] = 0
        eventtype: Optional[str] = 'user'
        timestart: Optional[int] = None
        timeduration: Optional[int] = 0
        visible: Optional[int] = 1
        sequence: Optional[int] = 1

    @dataclass
    class Delete:
        """Arg for Delete calendar events
        Constructor arguments:
        params: eventid (int): Event ID
        params: repeat (int): Delete comeplete series if repeated event
        """
        eventid: int
        repeat: int

    @dataclass
    class Details:
        """Event details
        Constructor arguments:
        params: eventids (List[int]): List of event ids
        params: courseids (List[int]): List of course ids for which events will be returned
        params: groupids (List[int]): List of group ids for which events should be returned
        params: categoryids (List[int]): List of category ids for which events will be returned
        """
        eventids: List[int]
        courseids: List[int]
        groupids: List[int]
        categoryids: List[int]

    @dataclass
    class Options:
        """Options
        params: userevents (int): Set to true to return current user's user events, Default for "1"
        params: siteevents (int): Set to true to return global events, Default for "1"
        params: timestart (int): Time from which events should be returned, Default for "0"
        params: timeend (int): Time to which the events should be returned. We treat 0 and null as no end, Default for "0"
        params: ignorehidden (int): Ignore hidden events or not, Default for "1"
        """
        userevents: Optional[int] = 1
        siteevents: Optional[int] = 1
        timestart: Optional[int] = 0
        timeend: Optional[int] = 0
        ignorehidden: Optional[int] = 1



from dataclasses import dataclass
from typing import Optional, List, Iterator
from moodle import Warning


@dataclass
class BagdeEndorsement:
    id: int  # Endorsement id
    badgeid: int  # Badge id
    issuername: str  # Endorsement issuer name
    issuerurl: str  # Endorsement issuer URL
    issueremail: str  # Endorsement issuer email
    claimid: str  # Claim URL
    claimcomment: str  # Claim comment
    dateissued: int  # Date issued


@dataclass
class BagdeAlignment:
    id: Optional[int]  # Alignment id
    badgeid: Optional[int]  # Badge id
    targetName: Optional[str]  # Target name
    targetUrl: Optional[str]  # Target URL
    targetDescription: Optional[str]  # Target description
    targetFramework: Optional[str]  # Target framework
    targetCode: Optional[str]  # Target code


@dataclass
class RelatedBadge:
    id: int  # Badge id
    name: str  # Badge name
    version: Optional[str]  # Version
    language: Optional[str]  # Language
    type: Optional[int]  # Type


@dataclass
class Badge:
    id: Optional[int]  # Badge id
    name: str  # Badge name
    description: str  # Badge description
    timecreated: Optional[int]  # Time created
    timemodified: Optional[int]  # Time modified
    usercreated: Optional[int]  # User created
    usermodified: Optional[int]  # User modified
    issuername: str  # Issuer name
    issuerurl: str  # Issuer URL
    issuercontact: str  # Issuer contact
    expiredate: Optional[int]  # Expire date
    expireperiod: Optional[int]  # Expire period
    type: Optional[int]  # Type
    courseid: Optional[int]  # Course id
    message: Optional[str]  # Message
    messagesubject: Optional[str]  # Message subject
    attachment: Optional[int]  # Attachment
    notification: Optional[int]  # Whether to notify when badge is awarded
    nextcron: Optional[int]  # Next cron
    status: Optional[int]  # Status
    issuedid: Optional[int]  # Issued id
    uniquehash: str  # Unique hash
    dateissued: int  # Date issued
    dateexpire: Optional[int]  # Date expire
    visible: Optional[int]  # Visible
    email: Optional[str]  # User email
    version: Optional[str]  # Version
    language: Optional[str]  # Language
    imageauthorname: Optional[str]  # Name of the image author
    imageauthoremail: Optional[str]  # Email of the image author
    imageauthorurl: Optional[str]  # URL of the image author
    imagecaption: Optional[str]  # Caption of the image
    badgeurl: str  # Badge URL
    endorsement: Optional[BagdeEndorsement]  # Badge endorsement
    alignment: List[BagdeAlignment]  # Badge alignments
    relatedbadges: List[RelatedBadge]  # Related badges


@dataclass
class BadgeResponse:
    badges: List[Badge]
    warnings: List[Warning]  # list of warnings

    def __getitem__(self, index: int) -> Badge:
        return self.badges[index]

    def __iter__(self) -> Iterator[Badge]:
        return iter(self.badges)

    def __len__(self) -> int:
        return len(self.badges)

    def __bool__(self) -> bool:
        return bool(self.badges)



from dataclasses import dataclass
from typing import List, Optional
from moodle import ResponsesFactory, Warning


@dataclass
class File:
    """File
    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Config:
    """Config
    Args
        id (Optional[int]): assign_plugin_config id
        assignment (Optional[int]): assignment id
        plugin (str): plugin
        subtype (str): subtype
        name (str): name
        value (str): value
    """
    id: Optional[int]
    assignment: Optional[int]
    plugin: str
    subtype: str
    name: str
    value: str


@dataclass
class Assignment:
    """Assigment
    Args:
        id (int): assignment id
        cmid (int): course module id
        course (int): course id
        name (str): assignment name
        nosubmissions (int): no submissions
        submissiondrafts (int): submissions drafts
        sendnotifications (int): send notifications
        sendlatenotifications (int): send notifications
        sendstudentnotifications (int): send student notifications (default)
        duedate (int): assignment due date
        allowsubmissionsfromdate (int): allow submissions from date
        grade (int): grade type
        timemodified (int): last time assignment was modified
        completionsubmit (int): if enabled, set activity as complete following submission
        cutoffdate (int): date after which submission is not accepted without an extension
        gradingduedate (int): the expected date for marking the submissions
        teamsubmission (int): if enabled, students submit as a team
        requireallteammemberssubmit (int): if enabled, all team members must submit
        teamsubmissiongroupingid (int): the grouping id for the team submission groups
        blindmarking (int): if enabled, hide identities until reveal identities actioned
        hidegrader (int): If enabled, hide grader to student
        revealidentities (int): show identities for a blind marking assignment
        attemptreopenmethod (str): method used to control opening new attempts
        maxattempts (int): maximum number of attempts allowed
        markingworkflow (int): enable marking workflow
        markingallocation (int): enable marking allocation
        requiresubmissionstatement (int): student must accept submission statement
        preventsubmissionnotingroup (Optional[int]): Prevent submission not in group
        submissionstatement (Optional[str]): Submission statement formatted.
        submissionstatementformat (int): submissionstatement format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        configs (Config): configuration settings
        intro (Optional[str]): assignment intro, not allways returned because it deppends on the activity configuration
        introformat (Optional[int]): intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        introfiles (List[File]): Files in the introduction text
        introattachments (List[File]): intro attachments files
    """
    id: int
    cmid: int
    course: int
    name: str
    nosubmissions: int
    submissiondrafts: int
    sendnotifications: int
    sendlatenotifications: int
    sendstudentnotifications: int
    duedate: int
    allowsubmissionsfromdate: int
    grade: int
    timemodified: int
    completionsubmit: int
    cutoffdate: int
    gradingduedate: int
    teamsubmission: int
    requireallteammemberssubmit: int
    teamsubmissiongroupingid: int
    blindmarking: int
    hidegrader: int
    revealidentities: int
    attemptreopenmethod: str
    maxattempts: int
    markingworkflow: int
    markingallocation: int
    requiresubmissionstatement: int
    preventsubmissionnotingroup: Optional[int]
    submissionstatement: Optional[str]
    submissionstatementformat: int
    configs: Config
    intro: Optional[str]
    introformat: Optional[int]
    introfiles: List[File]
    introattachments: List[File]


@dataclass
class AssignmentCourse(ResponsesFactory[Assignment]):
    """Courses with assignment
    Args:
        id (int): course id
        fullname (str): course full name
        shortname (str): course short name
        timemodified (int): last time modified
        assignments (List[Assignment]): assignment info
    """
    id: int
    fullname: str
    shortname: str
    timemodified: int
    assignments: List[Assignment]

    @property
    def items(self) -> List[Assignment]:
        return self.assignments


@dataclass
class Assignments(ResponsesFactory[AssignmentCourse]):
    """Assigments from get assignments
    Args:
        courses (List[AssignmentCourse]): List of course with assigments
        warnings (List[Warning]): List of warnings
    """
    courses: List[AssignmentCourse]
    warnings: List[Warning]

    @property
    def items(self) -> List[AssignmentCourse]:
        return self.courses



from dataclasses import dataclass, field
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class BlockFile:
    """Block file
    Constructor arguments:
    param: filename (Optional[str]): File name.
    param: filepath (Optional[str]): File path.
    param: filesize (Optional[int]): File size.
    param: fileurl (Optional[str]): Downloadable file url.
    param: timemodified (Optional[int]): Time modified.
    param: mimetype (Optional[str]): File mime type.
    param: isexternalfile (Optional[int]): Whether is an external file.
    param: repositorytype (Optional[str]): The repository type for external files
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class BlockContent:
    """Block content
    Constructor arguments:
    param: title (str) : Block title.
    param: content (str) : Block contents.
    param: contentformat (int) : content format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    param: footer (str) : Block footer.
    param: files (List[BlockFile]) : Block files.
    """
    title: str
    content: str
    contentformat: int
    footer: Optional[str]
    files: List[BlockFile] = field(default_factory=list)

    def __str__(self) -> str:
        return self.title


@dataclass
class Block:
    """Block information
    Constructor arguments:
    params: instanceid (int): Block instance id.
    params: name (str): Block name.
    params: region (str): Block region.
    params: positionid (int): Position id.
    params: collapsible (int): Whether the block is collapsible.
    params: dockable (int): Whether the block is dockable.
    params: weight (Optional[int]): Used to order blocks within a region.
    params: visible (Optional[int]): Whether the block is visible.
    params: contents (BlockContent): Block contents (if required).
    """
    instanceid: int
    name: str
    region: str
    positionid: Optional[int]
    collapsible: bool
    dockable: bool
    weight: Optional[int]
    visible: bool
    contents: Optional[BlockContent]

    def __str__(self) -> str:
        return self.name


@dataclass
class Blocks(ResponsesFactory[Block]):
    """Blocks information for a course.
    params: blocks (List[Block]): List of blocks in the course.
    params: warnings (List[Warning]): warning
    """
    blocks: List[Block] = field(default_factory=list)
    warnings: List[Warning] = field(default_factory=list)

    @property
    def items(self) -> List[Block]:
        return self.blocks



from dataclasses import dataclass, field
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class BlogSummaryFiles:
    """Summary file
    Constructor arguments:
    param: filename (Optional[str]): File name.
    param: filepath (Optional[str]): File path.
    param: filesize (Optional[int]): File size.
    param: fileurl (Optional[str]): Downloadable file url.
    param: timemodified (Optional[int]): Time modified.
    param: mimetype (Optional[str]): File mime type.
    param: isexternalfile (Optional[int]): Whether is an external file.
    param: repositorytype (Optional[str]): The repository type for the external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class BlogAttachmentFile:
    """Attachment File
    Constructor arguments:
    param: filename: (Optional[str]): File name.
    param: filepath: (Optional[str]): File path.
    param: filesize: (Optional[int]): File size.
    param: fileurl: (Optional[str]): Downloadable file url.
    param: timemodified: (Optional[int]): Time modified.
    param: mimetype: (Optional[str]): File mime type.
    param: isexternalfile: (Optional[int]): Whether is an external file.
    param: repositorytype: (Optional[str]): The repository type for the external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class BlogTag:
    """Tag
    Constructor arguments:
    param: id: (int): Tag id.
    param: name: (str): Tag name.
    param: rawname: (str): The raw, unnormalised name for the tag as entered by users.
    param: isstandard: (int): Whether this tag is standard.
    param: tagcollid: (int): Tag collection id.
    param: taginstanceid: (int): Tag instance id.
    param: taginstancecontextid: (int): Context the tag instance belongs to.
    param: itemid: (int): Id of the record tagged.
    param: ordering: (int): Tag ordering.
    param: flag: (int): Whether the tag is flagged as inappropriate.
    """
    id: int
    name: str
    rawname: str
    isstandard: int
    tagcollid: int
    taginstanceid: int
    taginstancecontextid: int
    itemid: int
    ordering: int
    flag: int


@dataclass
class BlogEntry:
    """Blog Entry
    Constructor arguments:
    params: id: (int): Post/entry id.
    params: module: (str): Where it was published the post (blog, blog_external...).
    params: userid: (int): Post author.
    params: courseid: (int): Course where the post was created.
    params: groupid: (int): Group post was created for.
    params: moduleid: (int): Module id where the post was created (not used anymore).
    params: coursemoduleid: (int): Course module id where the post was created.
    params: subject: (str): Post subject.
    params: summary: (str): Post summary.
    params: summaryformat: (int): Default for "1" # summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: content: (str): Post content.
    params: uniquehash: (str): Post unique hash.
    params: rating: (int): Post rating.
    params: format: (int): Post content format.
    params: attachment: (str): Post atachment.
    params: publishstate: (str): Post publish state.
    params: lastmodified: (int): When it was last modified.
    params: created: (int): When it was created.
    params: usermodified: (int): User that updated the post.
    params: summaryfiles: (List[BlogSummaryFiles]): summaryfiles
    params: attachmentfiles: (List[BlogAttachmentFile]): attachmentfiles
    params: tags: (List[BlogTag]): Tags.
    """
    id: int
    module: str
    userid: int
    courseid: int
    groupid: int
    moduleid: int
    coursemoduleid: int
    subject: str
    summary: str
    summaryformat: int
    content: Optional[str]
    uniquehash: str
    rating: int
    format: int
    attachment: str
    publishstate: str
    lastmodified: int
    created: int
    usermodified: Optional[int]
    summaryfiles: List[BlogSummaryFiles] = field(default_factory=list)
    attachmentfiles: List[BlogAttachmentFile] = field(default_factory=list)
    tags: List[BlogTag] = field(default_factory=list)


@dataclass
class BlogEntries(ResponsesFactory[BlogEntry]):
    """Blog entries.
    Constructor arguments:
    params: entries: (List[BlogEntry]): list of BlogEntry.
    params: totalentries: (int): The total number of entries found.
    params: warnings: (List[Warning]): list of warnings.

    Returns:
        BlogEntries: Returns blog entries.
    """
    entries: List[BlogEntry] = field(default_factory=list)
    totalentries: int = 0
    warnings: List[Warning] = field(default_factory=list)

    @property
    def items(self) -> List[BlogEntry]:
        return self.entries

    @dataclass
    class Filter:
        """Parameters to filter blog listings.
        Constructor arguments:
        params: name (str): The expected keys (value format) are:
                                tag      PARAM_NOTAGS blog tag
                                tagid    PARAM_INT    blog tag id
                                userid   PARAM_INT    blog author (userid)
                                cmid    PARAM_INT    course module id
                                entryid  PARAM_INT    entry id
                                groupid  PARAM_INT    group id
                                courseid PARAM_INT    course id
                                search   PARAM_RAW    search term
        params: value (str): The value of the filter.
        """
        name: str
        value: str



from datetime import datetime
from typing import List, Optional, Union
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import (
    Assignments,
    Grades,
    Participant,
    PluginData,
    AdvanceGradingData,
    UserFlag,
    View,
)


class BaseAssign(BaseMoodle):
    def get_assignments(
            self,
            courseids: Optional[List[int]] = None,
            capabilities: Optional[List[str]] = None,
            includenotenrolledcourses: Optional[int] = None) -> Assignments:
        """Returns the courses and assignments for the users capability

        Args:
            courseids (Optional[List[int]], optional): 0 or more course ids. Defaults to None.
            capabilities (Optional[List[str]], optional): list of capabilities used to filter courses. Defaults to None.
            includenotenrolledcourses (Optional[int], optional): whether to return courses that the user can see even if is not enroled in. This requires the parameter courseids to not be empty. Defaults to None.

        Returns:
            Assignments: Returns the courses and assignments for the users capability
        """
        res = self.moodle.post(
            'mod_assign_get_assignments',
            courseids=courseids,
            capabilities=capabilities,
            includenotenrolledcourses=includenotenrolledcourses,
        )
        return from_dict(Assignments, res)

    def get_grades(self,
                   assignmentids: List[int],
                   since: Union[datetime, int] = 0) -> Grades:
        """Returns grades from the assignment

        Args:
            assignmentids (List[int]): 1 or more assignment ids
            since (Union[datetime, int], optional):  timestamp, only return records where timemodified >= since. Defaults to 0.

        Returns:
            Grades: Returns grades from the assignment
        """
        res = self.moodle.post(
            'mod_assign_get_grades',
            assignmentids=assignmentids,
            since=since,
        )
        return from_dict(Grades, res)

    def list_participants(
            self,
            assignid: int,
            groupid: int,
            filter: str,
            skip: int = 0,
            limit: int = 0,
            onlyids: Optional[int] = None,
            includeenrolments: int = 1,
            tablesort: Optional[int] = None) -> List[Participant]:
        """List the participants for a single assignment, with some summary info about their submissions.

        Args:
            assignid (int): assign instance id
            groupid (int): group id
            filter (str): search string to filter the results
            skip (int, optional): number of records to skip. Defaults to 0.
            limit (int, optional): maximum number of records to return. Defaults to 0.
            onlyids (Optional[int], optional): Do not return all user fields. Defaults to None.
            includeenrolments (int, optional): Do return courses where the user is enrolled. Defaults to 1.
            tablesort (Optional[int], optional): Apply current user table sorting preferences.. Defaults to None.

        Returns:
            List[Participant]: list of participant
        """
        res = self.moodle.post(
            'mod_assign_list_participants',
            assignid=assignid,
            groupid=groupid,
            filter=filter,
            skip=skip,
            limit=limit,
            onlyids=onlyids,
            includeenrolments=includeenrolments,
            tablesort=tablesort,
        )
        return [from_dict(Participant, data) for data in res] if res else []

    def lock_submissions(self, assignmentid: int,
                         userids: List[int]) -> List[Warning]:
        res = self.moodle.post(
            'mod_assign_lock_submissions',
            assignmentid=assignmentid,
            userids=userids,
        )
        return [from_dict(Warning, data) for data in res] if res else []

    def reveal_identities(self, assignmentid: int) -> List[Warning]:
        res = self.moodle.post(
            'mod_assign_reveal_identities',
            assignmentid=assignmentid,
        )
        return [from_dict(Warning, data) for data in res] if res else []

    def revert_submissions_to_draft(self, assignmentid: int,
                                    userids: List[int]) -> List[Warning]:
        res = self.moodle.post(
            'mod_assign_revert_submissions_to_draft',
            assignmentid=assignmentid,
            userids=userids,
        )
        return [from_dict(Warning, data) for data in res] if res else []

    def save_grade(
            self,
            assignmentid: int,
            userid: int,
            grade: float,
            attemptnumber: int,
            addattempt: int,
            workflowstate: str,
            applytoall: int,
            plugindata: Optional[PluginData] = None,
            advancedgradingdata: Optional[AdvanceGradingData] = None) -> None:
        self.moodle.post(
            'mod_assign_save_grade',
            assignmentid=assignmentid,
            userid=userid,
            grade=grade,
            attemptnumber=attemptnumber,
            addattempt=addattempt,
            workflowstate=workflowstate,
            applytoall=applytoall,
            plugindata=plugindata,
            advancedgradingdata=advancedgradingdata,
        )

    def save_grades(self, assignmentid: int, applytoall: int,
                    grades: List[Grades.Create]) -> None:
        self.moodle.post('mod_assign_save_grades',
                         assignmentid=assignmentid,
                         applytoall=applytoall,
                         grades=grades)

    def save_submission(self, assignmentid: int,
                        plugindata: PluginData) -> List[Warning]:
        res = self.moodle.post('mod_assign_save_submission',
                               assignmentid=assignmentid,
                               plugindata=plugindata)
        return [from_dict(Warning, data) for data in res] if res else []

    def save_user_extensions(
            self, assignmentid: int, userids: List[int],
            dates: List[Union[datetime, int]]) -> List[Warning]:
        res = self.moodle.post('mod_assign_save_user_extensions',
                               assignmentid=assignmentid,
                               userids=userids,
                               dates=dates)
        return [from_dict(Warning, data) for data in res] if res else []

    def set_user_flags(self, assignmentid: int,
                       userflags: List[UserFlag]) -> List[UserFlag.Result]:
        res = self.moodle.post('mod_assign_set_user_flags',
                               assignmentid=assignmentid,
                               userflags=userflags)
        return [from_dict(UserFlag.Result, data)
                for data in res] if res else []

    def submit_for_grading(self, assignmentid: int,
                           acceptsubmissionstatement: int) -> List[Warning]:
        """Submit the current students assignment for grading

        Args:
            assignmentid (int): The assignment id to operate on
            acceptsubmissionstatement (int): Accept the assignment submission statement

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post(
            'mod_assign_submit_for_grading',
            assignmentid=assignmentid,
            acceptsubmissionstatement=acceptsubmissionstatement)
        return [from_dict(Warning, data) for data in res] if res else []

    def submit_grading_form(self, assignmentid: int, userid: int,
                            jsonformdata: str) -> List[Warning]:
        """Submit the grading form data via ajax

        Args:
            assignmentid (int): The assignment id to operate on
            userid (int): The user id the submission belongs to
            jsonformdata (str): The data from the grading form, encoded as a json array

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post('mod_assign_submit_grading_form')
        return [from_dict(Warning, data) for data in res] if res else []

    def unlock_submissions(self, assignmentid: int,
                           userids: int) -> List[Warning]:
        res = self.moodle.post('mod_assign_unlock_submissions')
        return [from_dict(Warning, data) for data in res] if res else []

    def view_assign(self, assignid: int) -> View:
        res = self.moodle.post('mod_assign_view_assign', assignid=assignid)
        return from_dict(View, res)

    def view_grading_table(self, assignid: int) -> View:
        res = self.moodle.post('mod_assign_view_grading_table')
        return from_dict(View, res)

    def view_submission_status(self, assignid: int) -> View:
        res = self.moodle.post('mod_assign_view_submission_status')
        return from_dict(View, res)

    def get_submissions(self):
        # TODO mod_assign_get_submissions
        res = self.moodle.post('mod_assign_get_submissions')
        return res

    def get_user_flags(self):
        # TODO mod_assign_get_user_flags
        res = self.moodle.post('mod_assign_get_user_flags')
        return res

    def get_user_mappings(self):
        # TODO mod_assign_get_user_mappings
        res = self.moodle.post('mod_assign_get_user_mappings')
        return res

    def get_submission_status(self):
        # TODO mod_assign_get_submission_status
        res = self.moodle.post('mod_assign_get_submission_status')
        return res

    def get_participant(self):
        # TODO mod_assign_get_participant
        res = self.moodle.post('mod_assign_get_participant')
        return res



from moodle import BaseMoodle
from . import BaseMobile


class Tool(BaseMoodle):
    def __post_init__(self, moodle) -> None:
        self._mobile = BaseMobile(moodle)

    @property
    def mobile(self) -> BaseMobile:
        return self._mobile



from dataclasses import dataclass


@dataclass
class MobilePlugin:
    component: str
    version: str
    addon: str
    dependencies: list
    fileurl: str
    filehash: str
    filesize: int
    handlers: str
    lang: str



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['moodle',
 'moodle.auth',
 'moodle.auth.email',
 'moodle.base',
 'moodle.base.preference',
 'moodle.core',
 'moodle.core.badges',
 'moodle.core.block',
 'moodle.core.blog',
 'moodle.core.calendar',
 'moodle.core.cohort',
 'moodle.core.comment',
 'moodle.core.competency',
 'moodle.core.course',
 'moodle.core.message',
 'moodle.core.notes',
 'moodle.core.user',
 'moodle.core.webservice',
 'moodle.mod',
 'moodle.mod.assign',
 'moodle.mod.forum',
 'moodle.tool',
 'moodle.tool.mobile',
 'moodle.utils']

package_data = \
{'': ['*']}

install_requires = \
['dacite>=1.5.1,<2.0.0', 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['moodle = moodle.__main__:main']}

setup_kwargs = {
    'name': 'moodlepy',
    'version': '0.8.1',
    'description': 'Client for moodle webservice',
    'long_description': "# moodlepy\n\n[![moodlepy - PyPi](https://img.shields.io/pypi/v/moodlepy)](https://pypi.org/project/moodlepy/)\n[![codecov](https://codecov.io/gh/hexatester/moodlepy/branch/master/graph/badge.svg)](https://codecov.io/gh/hexatester/moodlepy)\n[![BUILD](https://img.shields.io/travis/com/hexatester/moodlepy)](https://travis-ci.com/github/hexatester/moodlepy)\n[![LICENSE](https://img.shields.io/github/license/hexatester/moodlepy)](https://github.com/hexatester/moodlepy/blob/master/LICENSE)\n\nPython client for moodle webservice\n\n## Introduction\n\nThis library provide a pure Python interface for [Moodle Web Service](https://docs.moodle.org/dev/Web_services). It's compatible with Python versions 3.7+\n\n## Installing\n\nYou can install or upgrade moodlepy with:\n\n```bash\npip install moodlepy --upgrade\n```\n\nOr you can install from source with:\n\n```bash\ngit clone https://github.com/hexatester/moodlepy\ncd moodlepy\npython setup.py install\n```\n\n## Usage\n\nExample usage\n\n```python\nfrom moodle import Moodle\nurl = 'https://my.domain/webservice/rest/server.php'\ntoken = 'super secret token'\nmoodle = Moodle(url, token)\nraw_site_info = moodle('core_webservice_get_site_info')\nsite_info = moodle.core.webservice.get_site_info()  # return typed site_info\n\nprint(raw_site_info)\nprint(site_info)\n```\n\nIn the future all [Web service functions](https://docs.moodle.org/dev/Web_service_API_functions) will be covered by moodlepy\n\n# Moodle Web Service support\n\n❗️ Not all types and methods are supported, since moodlepy is not yet released.\nA = Added, W = Work In Progress\n\n| Area                 | Functions | Types | Tests | Status |\n| -------------------- | --------- | ----- | ----- | ------ |\n| auth_email           |           |       |       |        |\n| block                |           |       |       |        |\n| core_auth            |           |       |       |        |\n| core_backup          |           |       |       |        |\n| core_badge           | A         | A     | A     |        |\n| core_blog            | A         | A     | A     |        |\n| core_calendar        | A         | A     |       | W      |\n| core_cohort          |           |       |       |        |\n| core_comment         |           |       |       | W      |\n| core_competency      |           |       |       |        |\n| core_course          |           |       |       | W      |\n| core_customfield     |           |       |       |        |\n| core_enrol           |           |       |       |        |\n| core_fetch           |           |       |       | W      |\n| core_files           |           |       |       | W      |\n| core_filters         |           |       |       |        |\n| core_form            |           |       |       | W      |\n| core_get             |           |       |       | W      |\n| core_grade           |           |       |       | W      |\n| core_grades          |           |       |       |        |\n| core_grading         |           |       |       |        |\n| core_group           |           |       |       |        |\n| core_h5p             |           |       |       |        |\n| core_message         |           |       |       | W      |\n| core_notes           | A         | A     |       | W      |\n| core_output          |           |       |       |        |\n| core_question        | W         |       |       | W      |\n| core_rating          |           |       |       |        |\n| core_role            |           |       |       |        |\n| core_search          |           |       |       | W      |\n| core_session         |           |       |       | W      |\n| core_tag             |           |       |       |        |\n| core_update          |           |       |       |        |\n| core_user            |           |       |       | W      |\n| core_webservice      | A         | A     | A     | A      |\n| enrol_guest          |           |       |       |        |\n| enrol_manual         |           |       |       |        |\n| enrol_self           |           |       |       |        |\n| gradereport_overview |           |       |       |        |\n| gradereport_user     |           |       |       |        |\n| gradingform_guide    |           |       |       |        |\n| gradingform_rubric   |           |       |       |        |\n| local_mobile         |           |       |       | W      |\n| message_airnotifier  |           |       |       | W      |\n| message_popup        |           |       |       | W      |\n| mod_assign           |           |       |       | W      |\n| mod_book             |           |       |       |        |\n| mod_chat             |           |       |       | W      |\n| mod_choice           |           |       |       |        |\n| mod_data             |           |       |       |        |\n| mod_feedback         |           |       |       |        |\n| mod_folder           |           |       |       |        |\n| mod_forum            |           |       |       | W      |\n| mod_glossary         |           |       |       |        |\n| mod_imscp            |           |       |       |        |\n| mod_label            |           |       |       |        |\n| mod_lesson           |           |       |       | W      |\n| mod_lti              |           |       |       |        |\n| mod_page             |           |       |       | W      |\n| mod_quiz             |           |       |       | W      |\n| mod_resource         |           |       |       |        |\n| mod_scorm            |           |       |       |        |\n| mod_survey           |           |       |       | W      |\n| mod_url              |           |       |       | W      |\n| mod_wiki             |           |       |       |        |\n| mod_workshop         |           |       |       |        |\n| report_competency    |           |       |       |        |\n| report_insights      |           |       |       |        |\n| tool_analytics       |           |       |       |        |\n| tool_lp              |           |       |       |        |\n| tool_mobile          | A         | A     |       | W      |\n| tool_templatelibrary |           |       |       |        |\n| tool_usertours       |           |       |       |        |\n| tool_xmldb           |           |       |       |        |\n",
    'author': 'hexatester',
    'author_email': 'habibrohman@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hexatester/moodlepy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
from typing import List
from moodle import Warning


@dataclass
class View:
    """View Response
    Args:
        status (int): status: true if success
        warnings (List[Warning]): list of warnings
    """
    status: int
    warnings: List[Warning]



from dataclasses import dataclass
from typing import List
from moodle import ResponsesFactory, Warning


@dataclass
class NavigationOption:
    """Navigation Option
    Args:
        name (str): Option name
        available (int): Whether the option is available or not
    """
    name: str
    available: int


@dataclass
class CourseNavigation(ResponsesFactory[NavigationOption]):
    """Course
    Args:
        id (int): Course id
        options (List[NavigationOption]): list of NavigationOption
    """
    id: int
    options: List[NavigationOption]

    @property
    def items(self) -> List[NavigationOption]:
        return self.options


@dataclass
class NavigationOptions(ResponsesFactory[CourseNavigation]):
    """NavigationOptions
    Args:
        courses (List[CourseNavigation]): List of courses
        warnings (List[Warning]): list of warnings
    """
    courses: List[CourseNavigation]
    warnings: List[Warning]

    @property
    def items(self) -> List[CourseNavigation]:
        return self.courses
